/******************************************************************************
 *
 * AMDiS - Adaptive multidimensional simulations
 *
 * Copyright (C) 2013 Dresden University of Technology. All Rights Reserved.
 * Web: https://fusionforge.zih.tu-dresden.de/projects/amdis
 *
 * Authors: 
 * Simon Vey, Thomas Witkowski, Andreas Naumann, Simon Praetorius, et al.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 *
 * This file is part of AMDiS
 *
 * See also license.opensource.txt in the distribution.
 * 
 ******************************************************************************/


#include "VertexVector.h"
#include "DOFAdmin.h"
#include "DOFIterator.h"

namespace AMDiS {

  VertexVector::VertexVector(const DOFAdmin *a, std::string n)
    : DOFVectorDOF()
  {
    name = n;
    feSpace = NULL;
    admin = a;
    const_cast<DOFAdmin*>(admin)->addDOFIndexed(this);
    const_cast<DOFAdmin*>(admin)->addDOFContainer(this);
  }


  VertexVector::~VertexVector()
  {
    const_cast<DOFAdmin*>(admin)->removeDOFIndexed(this);
    const_cast<DOFAdmin*>(admin)->removeDOFContainer(this);
  }


  void VertexVector::set(DegreeOfFreedom val)
  {
    DOFIteratorBase it(const_cast<DOFAdmin*>(admin), USED_DOFS);
    for (it.reset(); !it.end(); ++it)
      if (!it.isDofFree()) 
	operator[](it.getDOFIndex()) = val;
  } 


  void VertexVector::print()
  {
    DOFIteratorBase it(const_cast<DOFAdmin*>(admin), USED_DOFS);
    for (it.reset(); !it.end(); ++it)
      if (!it.isDofFree())
	std::cout << "DOF " << it.getDOFIndex() << " -> " 
		  << operator[](it.getDOFIndex()) << "\n";
  }


  void VertexVector::changeDofIndices(std::map<DegreeOfFreedom, DegreeOfFreedom>& dofIndexMap)
  {
    std::vector<DegreeOfFreedom> tmp(vec.size());
    for (DofIndex::size_type i = 0; i < static_cast<DofIndex::size_type>(tmp.size()); i++)
      tmp[i] = i;
    
    for (std::map<DegreeOfFreedom, DegreeOfFreedom>::iterator it = dofIndexMap.begin(); 
	 it != dofIndexMap.end(); ++it)
      if (vec[it->first] == -1)
	tmp[it->second] = -1;
      else 
	tmp[it->second] = dofIndexMap[vec[it->first]];

    vec = tmp;
  }

}
